@@ -1127,11 +1127,6 @@ class TestCWD(_TestGlob):
 
         self.assert_equal(glob.glob('EF', root_dir=pathlib.Path(self.tempdir)), ['EF'])
 
-    def test_cwd_root_dir_pathlike_bytes(self):
-        """Test root level glob when we switch directory via `root_dir` with a path-like object."""
-
-        self.assert_equal(glob.glob(b'EF', root_dir=pathlib.Path(self.tempdir)), [b'EF'])
-
 
 class TestGlobCornerCase(_TestGlob):
     """
@@ -1213,6 +1208,17 @@ class TestGlobEscapes(unittest.TestCase):
                 )
             )
 
+    def test_raw_escape_deprecation(self):
+        """Test raw escape deprecation."""
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+
+            glob.raw_escape(r'test\\test')
+
+            self.assertTrue(len(w) == 1)
+            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
+
     def test_escape(self):
         """Test path escapes."""
 
@@ -1292,7 +1298,7 @@ class TestGlobEscapes(unittest.TestCase):
         check('\\\\*\\*\\*', r'\\\\\*\\\*\\\*', unix=True)
         check('//?/c:/?', r'//\?/c:/\?', unix=True)
         check('//*/*/*', r'//\*/\*/\*', unix=True)
-        check('//[^what]/name/temp', r'//\[\^what\]/name/temp', unix=True)
+        check('//[^what]/name/temp', r'//\[^what\]/name/temp', unix=True)
 
 
 @unittest.skipUnless(sys.platform.startswith('win'), "Windows specific test")
@@ -1463,3 +1469,55 @@ class TestExpansionLimit(unittest.TestCase):
 
         with self.assertRaises(_wcparse.PatternLimitException):
             list(glob.iglob('{1..11}', flags=glob.BRACE, limit=10))
+
+    def test_limit_split(self):
+        """Test split limit."""
+
+        with self.assertRaises(_wcparse.PatternLimitException):
+            list(glob.iglob('|'.join(['a'] * 11), flags=glob.SPLIT, limit=10))
+
+    def test_limit_brace_glob(self):
+        """Test limit with brace and split."""
+
+        with self.assertRaises(_wcparse.PatternLimitException):
+            list(
+                glob.iglob(
+                    '{{{},{}}}'.format('|'.join(['a'] * 6), '|'.join(['a'] * 5)),
+                    flags=glob.SPLIT | _wcparse.BRACE,
+                    limit=10
+                )
+            )
+
+    def test_limit_wrap(self):
+        """Test limit that wraps internally."""
+
+        with self.assertRaises(_wcparse.PatternLimitException):
+            list(
+                glob.iglob(
+                    ['|'.join(['a'] * 10), '|'.join(['a'] * 5)],
+                    flags=glob.SPLIT | _wcparse.BRACE,
+                    limit=10
+                )
+            )
+
+
+class TestInputTypes(unittest.TestCase):
+    """Test input types."""
+
+    def test_cwd_root_dir_pathlike_bytes_str(self):
+        """Test root level glob when we switch directory via `root_dir` with a path-like object."""
+
+        with self.assertRaises(TypeError):
+            glob.glob(b'docs/*', root_dir=pathlib.Path('.'))
+
+    def test_cwd_root_dir_bytes_str(self):
+        """Test root level glob when we switch directory via `root_dir` with a path-like object."""
+
+        with self.assertRaises(TypeError):
+            glob.glob(b'docs/*', root_dir='.')
+
+    def test_cwd_root_dir_str_bytes(self):
+        """Test root level glob when we switch directory via `root_dir` with a path-like object."""
+
+        with self.assertRaises(TypeError):
+            glob.glob('docs/*', root_dir=b'.')
