@@ -1526,19 +1526,65 @@ class TestGlobMatchSpecial(unittest.TestCase):
             glob.globmatch(pathlib.Path('markdown'), 'markdown', flags=glob.REALPATH, root_dir=pathlib.Path('docs/src'))
         )
 
-    def test_match_root_dir_pathlib_bytes(self):
-        """Test root directory with `globmatch` using `pathlib`."""
+    def test_match_pathlib_str_bytes(self):
+        """Test that mismatch type of `pathlib` and `bytes` asserts."""
 
         from wcmatch import pathlib
 
-        self.assertFalse(glob.globmatch(pathlib.Path('markdown'), b'markdown', flags=glob.REALPATH))
-        self.assertTrue(
+        with self.assertRaises(TypeError):
+            glob.globmatch(pathlib.Path('markdown'), b'markdown')
+
+    def test_match_str_bytes(self):
+        """Test that mismatch type of `str` and `bytes` asserts."""
+
+        with self.assertRaises(TypeError):
+            glob.globmatch('markdown', b'markdown')
+
+    def test_match_bytes_pathlib_str_realpath(self):
+        """Test that mismatch type of `pathlib` and bytes asserts."""
+
+        from wcmatch import pathlib
+
+        with self.assertRaises(TypeError):
             glob.globmatch(
                 pathlib.Path('markdown'),
-                b'markdown', flags=glob.REALPATH,
-                root_dir=pathlib.Path('docs/src')
+                b'markdown', flags=glob.REALPATH
+            )
+
+    def test_match_bytes_root_dir_pathlib_realpath(self):
+        """Test that mismatch type of root directory `pathlib` and `bytes` asserts."""
+
+        from wcmatch import pathlib
+
+        with self.assertRaises(TypeError):
+            glob.globmatch(
+                b'markdown',
+                b'markdown',
+                flags=glob.REALPATH,
+                root_dir=pathlib.Path('.')
+            )
+
+    def test_match_bytes_root_dir_str_realpath(self):
+        """Test that mismatch type of root directory `pathlib` and `bytes` asserts."""
+
+        with self.assertRaises(TypeError):
+            glob.globmatch(
+                b'markdown',
+                b'markdown',
+                flags=glob.REALPATH,
+                root_dir='.'
+            )
+
+    def test_match_str_root_dir_bytes_realpath(self):
+        """Test that mismatch type of root directory of `bytes` and `str` asserts."""
+
+        with self.assertRaises(TypeError):
+            glob.globmatch(
+                'markdown',
+                'markdown',
+                flags=glob.REALPATH,
+                root_dir=b'.'
             )
-        )
 
     def test_filter_root_dir_pathlib(self):
         """Test root directory with `globfilter`."""
@@ -1559,14 +1605,13 @@ class TestGlobMatchSpecial(unittest.TestCase):
 
         from wcmatch import pathlib
 
-        results = glob.globfilter(
-            [pathlib.Path('markdown')],
-            b'markdown',
-            flags=glob.REALPATH,
-            root_dir=pathlib.Path('docs/src')
-        )
-
-        self.assertTrue(all([isinstance(result, pathlib.Path) for result in results]))
+        with self.assertRaises(TypeError):
+            glob.globfilter(
+                [pathlib.Path('markdown')],
+                b'markdown',
+                flags=glob.REALPATH,
+                root_dir=pathlib.Path('docs/src')
+            )
 
 
 @skip_unless_symlink
@@ -1652,6 +1697,171 @@ class TestTilde(unittest.TestCase):
         self.assertNotEqual(len(files), len(gfiles))
 
 
+class TestIsMagic(unittest.TestCase):
+    """Test "is magic" logic."""
+
+    def test_default(self):
+        """Test default magic."""
+
+        self.assertTrue(glob.is_magic("test*"))
+        self.assertTrue(glob.is_magic("test["))
+        self.assertTrue(glob.is_magic("test]"))
+        self.assertTrue(glob.is_magic("test?"))
+        self.assertTrue(glob.is_magic("test\\"))
+
+        self.assertFalse(glob.is_magic("test~!()-/|{}"))
+
+    def test_extmatch(self):
+        """Test extended match magic."""
+
+        self.assertTrue(glob.is_magic("test*", flags=glob.EXTGLOB))
+        self.assertTrue(glob.is_magic("test[", flags=glob.EXTGLOB))
+        self.assertTrue(glob.is_magic("test]", flags=glob.EXTGLOB))
+        self.assertTrue(glob.is_magic("test?", flags=glob.EXTGLOB))
+        self.assertTrue(glob.is_magic("test\\", flags=glob.EXTGLOB))
+        self.assertTrue(glob.is_magic("test(", flags=glob.EXTGLOB))
+        self.assertTrue(glob.is_magic("test)", flags=glob.EXTGLOB))
+
+        self.assertFalse(glob.is_magic("test~!-/|{}", flags=glob.EXTGLOB))
+
+    def test_negate(self):
+        """Test negate magic."""
+
+        self.assertTrue(glob.is_magic("test*", flags=glob.NEGATE))
+        self.assertTrue(glob.is_magic("test[", flags=glob.NEGATE))
+        self.assertTrue(glob.is_magic("test]", flags=glob.NEGATE))
+        self.assertTrue(glob.is_magic("test?", flags=glob.NEGATE))
+        self.assertTrue(glob.is_magic("test\\", flags=glob.NEGATE))
+        self.assertTrue(glob.is_magic("test!", flags=glob.NEGATE))
+
+        self.assertFalse(glob.is_magic("test~()-/|{}", flags=glob.NEGATE))
+
+    def test_minusnegate(self):
+        """Test minus negate magic."""
+
+        self.assertTrue(glob.is_magic("test*", flags=glob.NEGATE | glob.MINUSNEGATE))
+        self.assertTrue(glob.is_magic("test[", flags=glob.NEGATE | glob.MINUSNEGATE))
+        self.assertTrue(glob.is_magic("test]", flags=glob.NEGATE | glob.MINUSNEGATE))
+        self.assertTrue(glob.is_magic("test?", flags=glob.NEGATE | glob.MINUSNEGATE))
+        self.assertTrue(glob.is_magic("test\\", flags=glob.NEGATE | glob.MINUSNEGATE))
+        self.assertTrue(glob.is_magic("test-", flags=glob.NEGATE | glob.MINUSNEGATE))
+
+        self.assertFalse(glob.is_magic("test~()!/|{}", flags=glob.NEGATE | glob.MINUSNEGATE))
+
+    def test_brace(self):
+        """Test brace magic."""
+
+        self.assertTrue(glob.is_magic("test*", flags=glob.BRACE))
+        self.assertTrue(glob.is_magic("test[", flags=glob.BRACE))
+        self.assertTrue(glob.is_magic("test]", flags=glob.BRACE))
+        self.assertTrue(glob.is_magic("test?", flags=glob.BRACE))
+        self.assertTrue(glob.is_magic("test\\", flags=glob.BRACE))
+        self.assertTrue(glob.is_magic("test{", flags=glob.BRACE))
+        self.assertTrue(glob.is_magic("test}", flags=glob.BRACE))
+
+        self.assertFalse(glob.is_magic("test~!-/|", flags=glob.BRACE))
+
+    def test_split(self):
+        """Test split magic."""
+
+        self.assertTrue(glob.is_magic("test*", flags=glob.SPLIT))
+        self.assertTrue(glob.is_magic("test[", flags=glob.SPLIT))
+        self.assertTrue(glob.is_magic("test]", flags=glob.SPLIT))
+        self.assertTrue(glob.is_magic("test?", flags=glob.SPLIT))
+        self.assertTrue(glob.is_magic("test\\", flags=glob.SPLIT))
+        self.assertTrue(glob.is_magic("test|", flags=glob.SPLIT))
+
+        self.assertFalse(glob.is_magic("test~()-!/", flags=glob.SPLIT))
+
+    def test_tilde(self):
+        """Test tilde magic."""
+
+        self.assertTrue(glob.is_magic("test*", flags=glob.GLOBTILDE))
+        self.assertTrue(glob.is_magic("test[", flags=glob.GLOBTILDE))
+        self.assertTrue(glob.is_magic("test]", flags=glob.GLOBTILDE))
+        self.assertTrue(glob.is_magic("test?", flags=glob.GLOBTILDE))
+        self.assertTrue(glob.is_magic("test\\", flags=glob.GLOBTILDE))
+        self.assertTrue(glob.is_magic("test~", flags=glob.GLOBTILDE))
+
+        self.assertFalse(glob.is_magic("test|()-!/", flags=glob.GLOBTILDE))
+
+    def test_all(self):
+        """Test tilde magic."""
+
+        flags = (
+            glob.EXTGLOB |
+            glob.NEGATE |
+            glob.BRACE |
+            glob.SPLIT |
+            glob.GLOBTILDE
+        )
+
+        self.assertTrue(glob.is_magic("test*", flags=flags))
+        self.assertTrue(glob.is_magic("test[", flags=flags))
+        self.assertTrue(glob.is_magic("test]", flags=flags))
+        self.assertTrue(glob.is_magic("test?", flags=flags))
+        self.assertTrue(glob.is_magic(r"te\\st", flags=flags))
+        self.assertTrue(glob.is_magic(r"te\st", flags=flags))
+        self.assertTrue(glob.is_magic("test!", flags=flags))
+        self.assertTrue(glob.is_magic("test|", flags=flags))
+        self.assertTrue(glob.is_magic("test(", flags=flags))
+        self.assertTrue(glob.is_magic("test)", flags=flags))
+        self.assertTrue(glob.is_magic("test{", flags=flags))
+        self.assertTrue(glob.is_magic("test}", flags=flags))
+        self.assertTrue(glob.is_magic("test~", flags=flags))
+        self.assertTrue(glob.is_magic("test-", flags=flags | glob.MINUSNEGATE))
+
+        self.assertFalse(glob.is_magic("test-", flags=flags))
+        self.assertFalse(glob.is_magic("test!", flags=flags | glob.MINUSNEGATE))
+
+    def test_all_bytes(self):
+        """Test tilde magic."""
+
+        flags = (
+            glob.EXTGLOB |
+            glob.NEGATE |
+            glob.BRACE |
+            glob.SPLIT |
+            glob.GLOBTILDE
+        )
+
+        self.assertTrue(glob.is_magic(b"test*", flags=flags))
+        self.assertTrue(glob.is_magic(b"test[", flags=flags))
+        self.assertTrue(glob.is_magic(b"test]", flags=flags))
+        self.assertTrue(glob.is_magic(b"test?", flags=flags))
+        self.assertTrue(glob.is_magic(rb"te\\st", flags=flags))
+        self.assertTrue(glob.is_magic(rb"te\st", flags=flags))
+        self.assertTrue(glob.is_magic(b"test!", flags=flags))
+        self.assertTrue(glob.is_magic(b"test|", flags=flags))
+        self.assertTrue(glob.is_magic(b"test(", flags=flags))
+        self.assertTrue(glob.is_magic(b"test)", flags=flags))
+        self.assertTrue(glob.is_magic(b"test{", flags=flags))
+        self.assertTrue(glob.is_magic(b"test}", flags=flags))
+        self.assertTrue(glob.is_magic(b"test~", flags=flags))
+        self.assertTrue(glob.is_magic(b"test-", flags=flags | glob.MINUSNEGATE))
+
+        self.assertFalse(glob.is_magic(b"test-", flags=flags))
+        self.assertFalse(glob.is_magic(b"test!", flags=flags | glob.MINUSNEGATE))
+
+    def test_win_path(self):
+        """Test windows path."""
+
+        flags = (
+            glob.EXTGLOB |
+            glob.NEGATE |
+            glob.FORCEWIN |
+            glob.GLOBTILDE
+        )
+
+        self.assertFalse(glob.is_magic('//?/UNC/server/*[]!|(){}~-/', flags=flags))
+        self.assertFalse(glob.is_magic('//?/UNC/server/*[]!|()~-/', flags=flags | glob.BRACE))
+        self.assertFalse(glob.is_magic('//?/UNC/server/*[]!(){}~-/', flags=flags | glob.SPLIT))
+
+        self.assertTrue(glob.is_magic('//?/UNC/server/*[]!|(){}|~-/', flags=flags | glob.BRACE))
+        self.assertTrue(glob.is_magic('//?/UNC/server/*[]!(){}|~-/', flags=flags | glob.SPLIT))
+        self.assertTrue(glob.is_magic(r'\\\\server\\mount\\', flags=flags))
+
+
 class TestExpansionLimit(unittest.TestCase):
     """Test expansion limits."""
 
